from __future__ import print_function
import numpy as np
from vtk import vtkXMLPImageDataReader
from vtk.util import numpy_support
import glob
import matplotlib
#matplotlib.use('TkAgg')
from matplotlib import pyplot as plt
from matplotlib import rc
from matplotlib.colors import SymLogNorm

img_file_name = 'orszag_tang.eps'
field_name = 'jz'

rc('text', usetex=True)
rc('font', family='serif')

fig,axs = plt.subplots(2, 3, figsize=(7.5,5.))

for i in range(3):
  axs[0,i].set_xlabel(r'$t / \Omega_{i,0}$')
  axs[1,i].set_xlabel(r'$x / d_{i,0}$')
axs[0,0].set_ylabel(r'Energy Change~/~$\mathcal{E}(t=0)$')
axs[1,0].set_ylabel(r'$y / d_{i,0}$')

axs[0,0].set_title(r'(a) Moment Fitting')
axs[0,1].set_title(r'(b) Standard PFC')
axs[0,2].set_title(r'(c) Fluid Gradient Closure')

axs[1,0].set_xticks([0,5,10,15,20,25])
axs[1,1].set_xticks([0,5,10,15,20,25])
axs[1,2].set_xticks([0,5,10,15,20,25])

#axs[1].set_yticklabels([])
#axs[2].set_yticklabels([])
#axs.set_xticklabels([])
#plt.colorbar(pad = 0.04)

reader = vtkXMLPImageDataReader()

# deltaf
loadpath = 'ot_vlasov_512x34xx_a1_m100_pfc_deltaf_mcorr_Qcweno_dp/output50.pvti'
print("Path:", loadpath)
reader.SetFileName(loadpath)
reader.Update()
img = reader.GetOutput()

dims = [img.GetDimensions()[0]-1,img.GetDimensions()[1]-1,img.GetDimensions()[2]-1]
origin = img.GetOrigin()
spacing = img.GetSpacing()

x = np.array([origin[0] + i*spacing[0] for i in range(dims[0])])
y = np.array([origin[1] + i*spacing[1] for i in range(dims[1])])
z = numpy_support.vtk_to_numpy(img.GetCellData().GetArray(field_name))
z = np.reshape(z,(dims[1],dims[0]))
z_min = -0.8 #-np.maximum(np.abs(z.min()), np.abs(z.max()))
z_max =  0.8 #-z_min

plt.axis([x.min(), x.max(), y.min(), y.max()])

im = axs[1,0].pcolor(x, y, z, cmap='coolwarm', vmin=z_min, vmax=z_max, rasterized=True, shading='auto')
#im = axs[1,0].pcolor(x, y, z, norm=SymLogNorm(linthresh=0.05, linscale=0.002, vmin=-1.5, vmax=1.5), cmap='coolwarm', shading='auto')


# PFC
loadpath = 'ot_vlasov_512x34xx_a1_m100_pfc_dp/output50.pvti'
print("Path:", loadpath)
reader.SetFileName(loadpath)
reader.Update()
img = reader.GetOutput()

dims = [img.GetDimensions()[0]-1,img.GetDimensions()[1]-1,img.GetDimensions()[2]-1]
origin = img.GetOrigin()
spacing = img.GetSpacing()

x = np.array([origin[0] + i*spacing[0] for i in range(dims[0])])
y = np.array([origin[1] + i*spacing[1] for i in range(dims[1])])
z = numpy_support.vtk_to_numpy(img.GetCellData().GetArray(field_name))
z = np.reshape(z,(dims[1],dims[0]))

im = axs[1,1].pcolor(x, y, z, cmap='coolwarm', vmin=z_min, vmax=z_max, rasterized=True, shading='auto')


# 10 mom
loadpath = 'ot_10mom_2048x_gradT/output50.pvti'
print("Path:", loadpath)
reader.SetFileName(loadpath)
reader.Update()
img = reader.GetOutput()

dims = [img.GetDimensions()[0]-1,img.GetDimensions()[1]-1,img.GetDimensions()[2]-1]
origin = img.GetOrigin()
spacing = img.GetSpacing()

x = np.array([origin[0] + i*spacing[0] for i in range(dims[0])])
y = np.array([origin[1] + i*spacing[1] for i in range(dims[1])])
z = numpy_support.vtk_to_numpy(img.GetCellData().GetArray(field_name))
z = np.reshape(z,(dims[1],dims[0]))

im = axs[1,2].pcolor(x, y, z, cmap='coolwarm', vmin=z_min, vmax=z_max, rasterized=True, shading='auto')

p0 = axs[1,0].get_position().get_points().flatten()
p1 = axs[1,1].get_position().get_points().flatten()
p2 = axs[1,2].get_position().get_points().flatten()
ax_cbar = fig.add_axes([p0[0]+0.028, -0.02, p2[2]-p0[0], 0.02])
fig.colorbar(im, cax=ax_cbar, orientation='horizontal')


######## total energy
#
output_directory_path = "ot_vlasov_512x34xx_a1_m100_pfc_deltaf_mcorr_Qcweno_dp"
path = output_directory_path + "/csv/*.csv"
data_sum = np.loadtxt(path[:-5]+"output_0_0_0.csv", delimiter=' ')
num_rows = data_sum.shape[0]
data_sum[:,1:] = 0.

files = glob.glob(path)
num_files = len(files)

i = 0
for fname in files:
  data = np.loadtxt(fname, delimiter=' ')
  num_rows = min(num_rows,data.shape[0])
  data_sum[:num_rows,1:] += data[:num_rows,1:]

time = data_sum[:num_rows,0]
electric_energy = data_sum[:num_rows,1]
magnetic_energy = data_sum[:num_rows,2]
kinetic_energy_e = data_sum[:num_rows,7]
thermal_energy_e = data_sum[:num_rows,8]
kinetic_energy_i = data_sum[:num_rows,13]
thermal_energy_i = data_sum[:num_rows,14]

energy = electric_energy[:] + magnetic_energy[:] + kinetic_energy_e[:] + \
          thermal_energy_e[:] + kinetic_energy_i[:] + thermal_energy_i[:]
im0 = axs[0,0].plot(time[:], (magnetic_energy[:]-magnetic_energy[0])/energy[0], color="black", linestyle="-")
im1 = axs[0,0].plot(time[:], (kinetic_energy_i[:]-kinetic_energy_i[0])/energy[0], color="orange", linestyle="-")
im2 = axs[0,0].plot(time[:], (thermal_energy_i[:]-thermal_energy_i[0])/energy[0], color="darkblue", linestyle="--")
im3 = axs[0,0].plot(time[:], (thermal_energy_e[:]-thermal_energy_e[0])/energy[0], color="red", linestyle="--")
axs[0,0].set_xlim([0.,time[len(time)-1]])
axs[0,0].set_ylim([-0.02,0.02])
axs[0,0].yaxis.grid(True)
#axs[0,0].legend([r'$\mathcal{E}_M-\mathcal{E}_M(t=0)$',r'$\mathcal{E}_{kin,i}-\mathcal{E}_{kin,i}(t=0)$',
#  r'$\mathcal{E}_{th,i}-\mathcal{E}_{th,i}(t=0)$',r'$\mathcal{E}_{th,e}-\mathcal{E}_{th,e}(t=0)$'], loc="best", prop={'size': 7.5})

#
output_directory_path = "ot_vlasov_512x34xx_a1_m100_pfc_dp"
path = output_directory_path + "/csv/*.csv"
data_sum = np.loadtxt(path[:-5]+"output_0_0_0.csv", delimiter=' ')
num_rows = data_sum.shape[0]
data_sum[:,1:] = 0.

files = glob.glob(path)
num_files = len(files)

i = 0
for fname in files:
  data = np.loadtxt(fname, delimiter=' ')
  num_rows = min(num_rows,data.shape[0])
  data_sum[:num_rows,1:] += data[:num_rows,1:]

time = data_sum[:num_rows,0]
electric_energy = data_sum[:num_rows,1]
magnetic_energy = data_sum[:num_rows,2]
kinetic_energy_e = data_sum[:num_rows,7]
thermal_energy_e = data_sum[:num_rows,8]
kinetic_energy_i = data_sum[:num_rows,13]
thermal_energy_i = data_sum[:num_rows,14]

energy = electric_energy[:] + magnetic_energy[:] + kinetic_energy_e[:] + \
          thermal_energy_e[:] + kinetic_energy_i[:] + thermal_energy_i[:]
axs[0,1].plot(time[:], (magnetic_energy[:]-magnetic_energy[0])/energy[0], color="black", linestyle="-")
axs[0,1].plot(time[:], (kinetic_energy_i[:]-kinetic_energy_i[0])/energy[0], color="orange", linestyle="-")
axs[0,1].plot(time[:], (thermal_energy_i[:]-thermal_energy_i[0])/energy[0], color="darkblue", linestyle="--")
axs[0,1].plot(time[:], (thermal_energy_e[:]-thermal_energy_e[0])/energy[0], color="red", linestyle="--")
axs[0,1].set_xlim([0.,time[len(time)-1]])
#axs[0,1].set_ylim([-0.02,0.8])
axs[0,1].yaxis.grid(True)
#axs[0,1].legend([r'$\mathcal{E}_{\mathrm{M}}-\mathcal{E}_{\mathrm{M,0}}$',r'$\mathcal{E}_{\mathrm{kin,i}}-\mathcal{E}_{\mathrm{kin,i,0}}$',
#  r'$\mathcal{E}_{\mathrm{th,i}}-\mathcal{E}_{\mathrm{th,i,0}}$',r'$\mathcal{E}_{\mathrm{th,e}}-\mathcal{E}_{\mathrm{th,e,0}}$'], loc="best", prop={'size': 7.5})
axs[0,1].legend(['Magnetic','Ion Kinetic','Ion Thermal','Electron Thermal'], loc="best", prop={'size': 7.5})

#
output_directory_path = "ot_10mom_2048x_gradT"
path = output_directory_path + "/csv/*.csv"
data_sum = np.loadtxt(path[:-5]+"output_0_0_0.csv", delimiter=' ')
num_rows = data_sum.shape[0]
data_sum[:,1:] = 0.

files = glob.glob(path)
num_files = len(files)

i = 0
for fname in files:
  data = np.loadtxt(fname, delimiter=' ')
  num_rows = min(num_rows,data.shape[0])
  data_sum[:num_rows,1:] += data[:num_rows,1:]

time = data_sum[:num_rows,0]
electric_energy = data_sum[:num_rows,1]
magnetic_energy = data_sum[:num_rows,2]
kinetic_energy_e = data_sum[:num_rows,7]
thermal_energy_e = data_sum[:num_rows,8]
kinetic_energy_i = data_sum[:num_rows,13]
thermal_energy_i = data_sum[:num_rows,14]

energy = electric_energy[:] + magnetic_energy[:] + kinetic_energy_e[:] + \
          thermal_energy_e[:] + kinetic_energy_i[:] + thermal_energy_i[:]
axs[0,2].plot(time[:], (magnetic_energy[:]-magnetic_energy[0])/energy[0], color="black", linestyle="-")
axs[0,2].plot(time[:], (kinetic_energy_i[:]-kinetic_energy_i[0])/energy[0], color="orange", linestyle="-")
axs[0,2].plot(time[:], (thermal_energy_i[:]-thermal_energy_i[0])/energy[0], color="darkblue", linestyle="--")
axs[0,2].plot(time[:], (thermal_energy_e[:]-thermal_energy_e[0])/energy[0], color="red", linestyle="--")
axs[0,2].set_xlim([0.,time[len(time)-1]])
axs[0,2].set_ylim([-0.02,0.02])
axs[0,2].yaxis.grid(True)
#axs[0,2].legend([r'$\mathcal{E}_M-\mathcal{E}_M(t=0)$',r'$\mathcal{E}_{kin,i}-\mathcal{E}_{kin,i}(t=0)$',
#  r'$\mathcal{E}_{th,i}-\mathcal{E}_{th,i}(t=0)$',r'$\mathcal{E}_{th,e}-\mathcal{E}_{th,e}(t=0)$'], loc="best", prop={'size': 7.5})


for j in range(3):
  axs[1,j].set_aspect("equal")
  axs[1,j].set_aspect("equal")
  axs[1,j].set_aspect("equal")


# write file
plt.tight_layout()
plt.savefig(img_file_name, bbox_inches='tight', dpi=300)

