from __future__ import print_function
import numpy as np
import matplotlib.pyplot as plt
import glob
import sys
from matplotlib.ticker import ScalarFormatter
from matplotlib import rc
from vtk import vtkXMLPImageDataReader
from vtk.util import numpy_support
import pyspedas
from pytplot import tplot
from datetime import datetime, timedelta
import datetime as dati
from spacepy.coordinates import Coords
from spacepy.time import Ticktock
from scipy import interpolate


rc('text', usetex=True)
rc('font', family='serif')

writepath = "foreshock_mms_data.eps"


fig, axs = plt.subplots(3, 2, figsize=(9.,6.))


### Transformation MMS (GSE) -> simulation coordinate system from Liu paper
# BL =  0.58*Bx + 0.24*By + 0.78*Bz
# BM =  0.64*Bx + 0.45*By - 0.62*Bz
# BN = -0.50*Bx + 0.85*By + 0.11*Bz


### MMS data


# magnetic field
mms_fgm = pyspedas.mms.fgm(trange=['2017-11-10/17:26:00', '2017-11-10/17:27:00'], data_rate='brst', notplot=True)

t_fgm = mms_fgm['mms1_fgm_b_gse_brst_l2']['x']
Bx = mms_fgm['mms1_fgm_b_gse_brst_l2']['y'][:,0]
By = mms_fgm['mms1_fgm_b_gse_brst_l2']['y'][:,1]
Bz = mms_fgm['mms1_fgm_b_gse_brst_l2']['y'][:,2]

t_fgm_conv = [datetime.fromtimestamp(elem) + timedelta(hours=-1) for elem in t_fgm]
print(t_fgm_conv[100])
BL =   0.58*Bx + 0.24*By + 0.78*Bz
BM =  -0.50*Bx + 0.85*By + 0.11*Bz
BN = -(0.64*Bx + 0.45*By - 0.62*Bz)


# electric field
mms_edp = pyspedas.mms.edp(trange=['2017-11-10/17:26:00', '2017-11-10/17:27:00'], data_rate='brst', notplot=True)

t_edp = mms_edp['mms1_edp_dce_gse_brst_l2']['x']
Ex = mms_edp['mms1_edp_dce_gse_brst_l2']['y'][:,0]
Ey = mms_edp['mms1_edp_dce_gse_brst_l2']['y'][:,1]
Ez = mms_edp['mms1_edp_dce_gse_brst_l2']['y'][:,2]
 
t_edp_conv = [datetime.fromtimestamp(elem) + timedelta(hours=-1) for elem in t_edp]
EL =   0.58*Ex + 0.24*Ey + 0.78*Ez
EM =  -0.50*Ex + 0.85*Ey + 0.11*Ez
EN = -(0.64*Ex + 0.45*Ey - 0.62*Ez)


# electron velocity and density
mms_des = pyspedas.mms.fpi(trange=['2017-11-10/17:26:00', '2017-11-10/17:27:00'], datatype='des-moms',
  data_rate='brst', center_measurement=True, notplot=True)

t_des_v = mms_des['mms1_des_bulkv_gse_brst']['x']
vex = mms_des['mms1_des_bulkv_gse_brst']['y'][:,0]
vey = mms_des['mms1_des_bulkv_gse_brst']['y'][:,1]
vez = mms_des['mms1_des_bulkv_gse_brst']['y'][:,2]

t_des_v_conv = [datetime.fromtimestamp(elem) + timedelta(hours=-1) for elem in t_des_v]
veL =   0.58*vex + 0.24*vey + 0.78*vez
veM =  -0.50*vex + 0.85*vey + 0.11*vez
veN = -(0.64*vex + 0.45*vey - 0.62*vez)



### simulation data

loadpath = 'liu_vlasov_1152x288x36xx_m100_gf-m1_deltaf_dp/output30.pvti'
reader = vtkXMLPImageDataReader()
print("Path: ", loadpath)

reader.SetFileName(loadpath)
reader.Update()
img = reader.GetOutput()

dims = [img.GetDimensions()[0]-1,img.GetDimensions()[1]-1,img.GetDimensions()[2]-1]
origin = img.GetOrigin()
spacing = img.GetSpacing()

x = np.array([origin[0] + i*spacing[0] for i in range(dims[0])])
y = np.array([origin[1] + i*spacing[1] for i in range(dims[1])])

ux_e = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('ux_e'))
uy_e = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('uy_e'))
uz_e = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('uz_e'))
Bx = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Bx'))
By = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('By'))
Bz = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Bz'))
Ex = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Ex'))
Ey = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Ey'))
Ez = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Ez'))

ux_e = np.reshape(ux_e,(dims[1],dims[0]))
uy_e = np.reshape(uy_e,(dims[1],dims[0]))
uz_e = np.reshape(uz_e,(dims[1],dims[0]))
Bx = np.reshape(Bx,(dims[1],dims[0]))
By = np.reshape(By,(dims[1],dims[0]))
Bz = np.reshape(Bz,(dims[1],dims[0]))
Ex = np.reshape(Ex,(dims[1],dims[0]))
Ey = np.reshape(Ey,(dims[1],dims[0]))
Ez = np.reshape(Ez,(dims[1],dims[0]))

num_line_points = 1000
line_beg_x = -1.
line_end_x = -1.
line_beg_y = -2.5
line_end_y = 2.5
line_x = np.linspace(line_beg_x, line_end_x, num_line_points)
line_y = np.linspace(line_beg_y, line_end_y, num_line_points)
path_length = np.sqrt( (line_end_x-line_beg_x)**2 + (line_end_y-line_beg_y)**2 )
r = np.linspace(0.,path_length,num_line_points)

f = interpolate.RectBivariateSpline(y,x,ux_e)
ux_e_sim = f(line_y, line_x)
ux_e_line = np.array([ux_e_sim[i,i] for i in range(num_line_points-1,-1,-1)])
f = interpolate.RectBivariateSpline(y,x,uy_e)
uy_e_sim = f(line_y, line_x)
uy_e_line = np.array([uy_e_sim[i,i] for i in range(num_line_points-1,-1,-1)])
f = interpolate.RectBivariateSpline(y,x,uz_e)
uz_e_sim = f(line_y, line_x)
uz_e_line = np.array([uz_e_sim[i,i] for i in range(num_line_points-1,-1,-1)])
f = interpolate.RectBivariateSpline(y,x,Ex)
Ex_sim = f(line_y, line_x)
Ex_line = np.array([Ex_sim[i,i] for i in range(num_line_points-1,-1,-1)])
f = interpolate.RectBivariateSpline(y,x,Ey)
Ey_sim = f(line_y, line_x)
Ey_line = np.array([Ey_sim[i,i] for i in range(num_line_points-1,-1,-1)])
f = interpolate.RectBivariateSpline(y,x,Ez)
Ez_sim = f(line_y, line_x)
Ez_line = np.array([Ez_sim[i,i] for i in range(num_line_points-1,-1,-1)])
f = interpolate.RectBivariateSpline(y,x,Bx)
Bx_sim = f(line_y, line_x)
Bx_line = np.array([Bx_sim[i,i] for i in range(num_line_points-1,-1,-1)])
f = interpolate.RectBivariateSpline(y,x,By)
By_sim = f(line_y, line_x)
By_line = np.array([By_sim[i,i] for i in range(num_line_points-1,-1,-1)])
f = interpolate.RectBivariateSpline(y,x,Bz)
Bz_sim = f(line_y, line_x)
Bz_line = np.array([Bz_sim[i,i] for i in range(num_line_points-1,-1,-1)])

### plotting

for i in range(3):
  axs[i,0].set_xlim([0., path_length])
  axs[i,1].set_xlim([dati.datetime(2017,11,10,17,26,17,400000), dati.datetime(2017,11,10,17,26,18,100000)])

for i in range(2):
  axs[i,0].xaxis.set_ticklabels([])
  axs[i,1].xaxis.set_ticklabels([])

axs[0,1].plot(t_des_v_conv,veL/1.e4, t_des_v_conv,veM/1.e4, t_des_v_conv,veN/1.e4)
axs[0,1].legend([r'$u_{e,L}$',r'$u_{e,M}$',r'$u_{e,N}$'], loc="upper left", prop={'size': 8.5})
axs[0,1].set_ylabel(r'$\mathbf{u} / \mathrm{(10^4 km/s)}$')
axs[0,1].set_ylim(-0.06,0.11)

axs[1,1].plot(t_fgm_conv,BL, t_fgm_conv,BM, t_fgm_conv,BN)
axs[1,1].legend([r'$B_L$',r'$B_M$',r'$B_N$'], loc="upper left", prop={'size': 8.5})
axs[1,1].set_ylim(-6,2.5)
axs[1,1].set_ylabel(r'$\mathbf{B} / \mathrm{nT}$')

axs[2,1].plot(t_edp_conv,EL, t_edp_conv,EM, t_edp_conv,EN)
axs[2,1].legend([r'$E_L$',r'$E_M$',r'$E_N$'], loc="upper left", prop={'size': 8.5})
axs[2,1].set_ylabel(r'$\mathbf{E} / (\mathrm{mV/m})$')
axs[2,1].set_ylim(-6.,9.)
axs[2,1].set_xlabel(r'Time / s from 17:26:17 UT on 10 Nov 2017')
axs[2,1].xaxis.set_ticklabels([0.4,0.5,0.6,0.7,0.8,0.9,1.0,1.1])

axs[0,0].plot(r,ux_e_line*0.1, r,uy_e_line*0.1, r,uz_e_line*0.1) # *0.1 to translate from v_A,i to v_A,e (sqrt(m_e)=0.1 as m_e = 0.01)
axs[0,0].legend([r'$u_{e,x}$',r'$u_{e,y}$',r'$u_{e,z}$'], loc="upper left", prop={'size': 8.5})
axs[0,0].set_ylabel(r'$\mathbf{u} / v_{A,e,0}$')
axs[0,0].set_ylim(-0.2,0.65)

axs[1,0].plot(r,Bx_line, r,By_line, r,Bz_line)
axs[1,0].legend([r'$B_x$',r'$B_y$',r'$B_z$'], loc="upper left", prop={'size': 8.5})
axs[1,0].set_ylabel(r'$\mathbf{B} / B_0$')
axs[1,0].set_ylim(-1.75,1.)

axs[2,0].plot(r,Ex_line, r,Ey_line, r,Ez_line)
axs[2,0].legend([r'$E_x$',r'$E_y$',r'$E_z$'], loc="upper left", prop={'size': 8.5})
axs[2,0].set_ylabel(r'$\mathbf{E} / (B_0\,v_{A,0})$')
axs[2,0].set_ylim(-1.5,3.)
axs[2,0].set_xlabel(r'$r / d_{i,0}$')

axs[0,0].set_title(r'(a) Simulation Data', fontsize=16)
axs[0,1].set_title(r'(b) MMS Measurements', fontsize=16)

plt.tight_layout()
plt.savefig(writepath, bbox_inches='tight', dpi=300)

