from __future__ import print_function
import numpy as np
import matplotlib.pyplot as plt
import glob
import sys
from matplotlib.ticker import ScalarFormatter
from matplotlib import rc
from vtk import vtkXMLPImageDataReader
from vtk.util import numpy_support

rc('text', usetex=True)
rc('font', family='serif')

writepath = "gem_jz.eps"


fig, axs = plt.subplots(3, 2, figsize=(7.5,6.5))

for j in range(2):
  axs[0,j].set_xlabel(r'$t / \Omega_{i,0}^{-1}$')
for i in range(1,3):
  for j in range(2):
    axs[i,j].set_xlabel(r'$x / d_{i,0}$')

axs[0,0].set_ylabel(r'$\psi / B_0 d_{i,0}$')
axs[0,1].set_ylabel(r'$\mathcal{E} / \mathcal{E}(t=0)$')
for i in range(1,3):
  for j in range(2):
    axs[i,j].set_ylabel(r'$y / d_{i,0}$')


#axs[2,0].set_xlabel(r'$x / d_{i,0}$')
#axs[2,1].set_xlabel(r'$x / d_{i,0}$')
#
#axs[1,0].set_ylabel(r'$y / d_{i,0}$')
#axs[2,0].set_ylabel(r'$y / d_{i,0}$')

#axs[1,0].xaxis.set_ticklabels([])
#axs[1,1].xaxis.set_ticklabels([])
#axs[1,1].yaxis.set_ticklabels([])
#axs[2,1].yaxis.set_ticklabels([])

######## pcolor

field_name = 'jz'
reader = vtkXMLPImageDataReader()


# pfc 18^3
loadpath = 'gem_vlasov_256x128x18xx_pfc/output37.pvti'
print("Path: ", loadpath)

reader.SetFileName(loadpath)
reader.Update()
img = reader.GetOutput()

dims = [img.GetDimensions()[0]-1,img.GetDimensions()[1]-1,img.GetDimensions()[2]-1]
origin = img.GetOrigin()
spacing = img.GetSpacing()

x = np.array([origin[0] + (i-dims[0])*spacing[0] for i in range(2*dims[0])])
y = np.array([origin[1] + i*spacing[1] for i in range(2*dims[1])])
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray(field_name))
data = np.reshape(data,(dims[1],dims[0]))
z = np.zeros([2*dims[1],2*dims[0]])
z[0:dims[1],dims[0]:2*dims[0]] = data[:,:]
z[dims[1]:2*dims[1],dims[0]:2*dims[0]] = np.flip(data[:,:],0)
z[0:dims[1],0:dims[0]] = np.flip(data[:,:],1)
z[dims[1]:2*dims[1],0:dims[0]] = np.flip(np.flip(data[:,:],1),0)
z_min = -2.5; #z.min()
z_max = 0.;  #z.max()

im = axs[2,1].pcolor(x, y, z, cmap='magma_r', vmin=z_min, vmax=z_max, rasterized=True, shading='auto')
#fig.colorbar(im, ax=axs[1,1], pad = 0.04, aspect=20)#, shrink = .7)


# deltaf 18^3
loadpath = 'gem_vlasov_256x128x18xx_pfc_deltaf/output44.pvti'
print("Path: ", loadpath)

reader.SetFileName(loadpath)
reader.Update()
img = reader.GetOutput()

dims = [img.GetDimensions()[0]-1,img.GetDimensions()[1]-1,img.GetDimensions()[2]-1]
origin = img.GetOrigin()
spacing = img.GetSpacing()

x = np.array([origin[0] + (i-dims[0])*spacing[0] for i in range(2*dims[0])])
y = np.array([origin[1] + i*spacing[1] for i in range(2*dims[1])])
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray(field_name))
data = np.reshape(data,(dims[1],dims[0]))
z = np.zeros([2*dims[1],2*dims[0]])
z[0:dims[1],dims[0]:2*dims[0]] = data[:,:]
z[dims[1]:2*dims[1],dims[0]:2*dims[0]] = np.flip(data[:,:],0)
z[0:dims[1],0:dims[0]] = np.flip(data[:,:],1)
z[dims[1]:2*dims[1],0:dims[0]] = np.flip(np.flip(data[:,:],1),0)
#z_min = z.min()
#z_max = z.max()

im = axs[2,0].pcolor(x, y, z, cmap='magma_r', vmin=z_min, vmax=z_max, rasterized=True, shading='auto')
#fig.colorbar(im, ax=axs[1,0], pad = 0.04, aspect=20)#, shrink = .7)


# pfc 32^3
loadpath = 'gem_vlasov_256x128x32xx_pfc/output38.pvti'
print("Path: ", loadpath)

reader.SetFileName(loadpath)
reader.Update()
img = reader.GetOutput()

dims = [img.GetDimensions()[0]-1,img.GetDimensions()[1]-1,img.GetDimensions()[2]-1]
origin = img.GetOrigin()
spacing = img.GetSpacing()

x = np.array([origin[0] + (i-dims[0])*spacing[0] for i in range(2*dims[0])])
y = np.array([origin[1] + i*spacing[1] for i in range(2*dims[1])])
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray(field_name))
data = np.reshape(data,(dims[1],dims[0]))
z = np.zeros([2*dims[1],2*dims[0]])
z[0:dims[1],dims[0]:2*dims[0]] = data[:,:]
z[dims[1]:2*dims[1],dims[0]:2*dims[0]] = np.flip(data[:,:],0)
z[0:dims[1],0:dims[0]] = np.flip(data[:,:],1)
z[dims[1]:2*dims[1],0:dims[0]] = np.flip(np.flip(data[:,:],1),0)
#z_min = z.min()
#z_max = z.max()

im = axs[1,1].pcolor(x, y, z, cmap='magma_r', vmin=z_min, vmax=z_max, rasterized=True, shading='auto')
#fig.colorbar(im, ax=axs[0,1], pad = 0.04, aspect=20)#, shrink = .7)


# deltaf 32^3
loadpath = 'gem_vlasov_256x128x32xx_pfc_deltaf/output42.pvti'
print("Path: ", loadpath)

reader.SetFileName(loadpath)
reader.Update()
img = reader.GetOutput()

dims = [img.GetDimensions()[0]-1,img.GetDimensions()[1]-1,img.GetDimensions()[2]-1]
origin = img.GetOrigin()
spacing = img.GetSpacing()

x = np.array([origin[0] + (i-dims[0])*spacing[0] for i in range(2*dims[0])])
y = np.array([origin[1] + i*spacing[1] for i in range(2*dims[1])])
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray(field_name))
data = np.reshape(data,(dims[1],dims[0]))
z = np.zeros([2*dims[1],2*dims[0]])
z[0:dims[1],dims[0]:2*dims[0]] = data[:,:]
z[dims[1]:2*dims[1],dims[0]:2*dims[0]] = np.flip(data[:,:],0)
z[0:dims[1],0:dims[0]] = np.flip(data[:,:],1)
z[dims[1]:2*dims[1],0:dims[0]] = np.flip(np.flip(data[:,:],1),0)
#z_min = z.min()
#z_max = z.max()

im = axs[1,0].pcolor(x, y, z, cmap='magma_r', vmin=z_min, vmax=z_max, rasterized=True, shading='auto')
#fig.colorbar(im, ax=axs[0,0], pad = 0.04, aspect=20)#, shrink = .7)

#p0 = axs[0,0].get_position().get_points().flatten()
#p1 = axs[0,1].get_position().get_points().flatten()
p2 = axs[2,0].get_position().get_points().flatten()
p3 = axs[2,1].get_position().get_points().flatten()
ax_cbar = fig.add_axes([p2[0], -0.02, p3[2]-p2[0], 0.015])
fig.colorbar(im, cax=ax_cbar, orientation='horizontal')


######## magnetic flux

loadpath = 'rec_rates_gem_vlasov_256x128x18xx_pfc.dat'
t_pfc_18,rec_pfc_18 = np.loadtxt(loadpath, usecols=(0,1), unpack=True, delimiter=' ')
loadpath = 'rec_rates_gem_vlasov_256x128x18xx_pfc_deltaf.dat'
t_df_18,rec_df_18 = np.loadtxt(loadpath, usecols=(0,1), unpack=True, delimiter=' ')
loadpath = 'rec_rates_gem_vlasov_256x128x32xx_pfc.dat'
t_pfc_32,rec_pfc_32 = np.loadtxt(loadpath, usecols=(0,1), unpack=True, delimiter=' ')
loadpath = 'rec_rates_gem_vlasov_256x128x32xx_pfc_deltaf.dat'
t_df_32,rec_df_32 = np.loadtxt(loadpath, usecols=(0,1), unpack=True, delimiter=' ')


axs[0,0].plot(t_df_32, rec_df_32, linestyle="-", color="black")
axs[0,0].plot(t_pfc_32, rec_pfc_32, linestyle="-.", color="darkblue")
axs[0,0].plot(t_df_18, rec_df_18, linestyle="--", color="gray")
axs[0,0].plot(t_pfc_18, rec_pfc_18, linestyle=":", color="orange")
axs[0,0].set_ylim([0.,5.])
axs[0,0].set_xlim([0.,40.])
axs[0,0].legend([r'Moment Fitting $32^3$',r'Standard PFC $32^3$',r'Moment Fitting $18^3$',r'Standard PFC $18^3$'], loc="upper left", prop={'size': 7.25})


######## total energy
#
output_directory_path = "gem_vlasov_256x128x32xx_pfc_deltaf"
path = output_directory_path + "/csv/*.csv"
data_sum = np.loadtxt(path[:-5]+"output_0_0_0.csv", delimiter=' ')
num_rows = data_sum.shape[0]
data_sum[:,1:] = 0.

files = glob.glob(path)
num_files = len(files)

i = 0
for fname in files:
  data = np.loadtxt(fname, delimiter=' ')
  num_rows = min(num_rows,data.shape[0])
  data_sum[:num_rows,1:] += data[:num_rows,1:]
  if (sys.version_info[0] > 2):
    print("Processing file", i, "out of", num_files, end="\r", flush=True) # python3
  else:
    print("Processing file", i, "out of", num_files, end="\r") # python2
  i += 1

print("\nProcessing completed")

time = data_sum[:num_rows,0]
electric_energy = data_sum[:num_rows,1]
magnetic_energy = data_sum[:num_rows,2]
kinetic_energy_e = data_sum[:num_rows,7]
thermal_energy_e = data_sum[:num_rows,8]
kinetic_energy_i = data_sum[:num_rows,13]
thermal_energy_i = data_sum[:num_rows,14]

energy = electric_energy[:] + magnetic_energy[:] + kinetic_energy_e[:] + \
          thermal_energy_e[:] + kinetic_energy_i[:] + thermal_energy_i[:]
axs[0,1].plot(time[:], energy[:]/energy[0], color="black", linestyle="-")

#
output_directory_path = "gem_vlasov_256x128x32xx_pfc"
path = output_directory_path + "/csv/*.csv"
data_sum = np.loadtxt(path[:-5]+"output_0_0_0.csv", delimiter=' ')
num_rows = data_sum.shape[0]
data_sum[:,1:] = 0.

files = glob.glob(path)
num_files = len(files)

i = 0
for fname in files:
  data = np.loadtxt(fname, delimiter=' ')
  num_rows = min(num_rows,data.shape[0])
  data_sum[:num_rows,1:] += data[:num_rows,1:]
  if (sys.version_info[0] > 2):
    print("Processing file", i, "out of", num_files, end="\r", flush=True) # python3
  else:
    print("Processing file", i, "out of", num_files, end="\r") # python2
  i += 1

print("\nProcessing completed")

time = data_sum[:num_rows,0]
electric_energy = data_sum[:num_rows,1]
magnetic_energy = data_sum[:num_rows,2]
kinetic_energy_e = data_sum[:num_rows,7]
thermal_energy_e = data_sum[:num_rows,8]
kinetic_energy_i = data_sum[:num_rows,13]
thermal_energy_i = data_sum[:num_rows,14]

energy = electric_energy[:] + magnetic_energy[:] + kinetic_energy_e[:] + \
          thermal_energy_e[:] + kinetic_energy_i[:] + thermal_energy_i[:]
axs[0,1].plot(time[:], energy[:]/energy[0], color="darkblue", linestyle="-.")

#
output_directory_path = "gem_vlasov_256x128x18xx_pfc_deltaf"
path = output_directory_path + "/csv/*.csv"
data_sum = np.loadtxt(path[:-5]+"output_0_0_0.csv", delimiter=' ')
num_rows = data_sum.shape[0]
data_sum[:,1:] = 0.

files = glob.glob(path)
num_files = len(files)

i = 0
for fname in files:
  data = np.loadtxt(fname, delimiter=' ')
  num_rows = min(num_rows,data.shape[0])
  data_sum[:num_rows,1:] += data[:num_rows,1:]
  if (sys.version_info[0] > 2):
    print("Processing file", i, "out of", num_files, end="\r", flush=True) # python3
  else:
    print("Processing file", i, "out of", num_files, end="\r") # python2
  i += 1

print("\nProcessing completed")

time = data_sum[:num_rows,0]
electric_energy = data_sum[:num_rows,1]
magnetic_energy = data_sum[:num_rows,2]
kinetic_energy_e = data_sum[:num_rows,7]
thermal_energy_e = data_sum[:num_rows,8]
kinetic_energy_i = data_sum[:num_rows,13]
thermal_energy_i = data_sum[:num_rows,14]

energy = electric_energy[:] + magnetic_energy[:] + kinetic_energy_e[:] + \
          thermal_energy_e[:] + kinetic_energy_i[:] + thermal_energy_i[:]
axs[0,1].plot(time[:], energy[:]/energy[0], color="gray", linestyle="--")

#
output_directory_path = "gem_vlasov_256x128x18xx_pfc"
path = output_directory_path + "/csv/*.csv"
data_sum = np.loadtxt(path[:-5]+"output_0_0_0.csv", delimiter=' ')
num_rows = data_sum.shape[0]
data_sum[:,1:] = 0.

files = glob.glob(path)
num_files = len(files)

i = 0
for fname in files:
  data = np.loadtxt(fname, delimiter=' ')
  num_rows = min(num_rows,data.shape[0])
  data_sum[:num_rows,1:] += data[:num_rows,1:]
  if (sys.version_info[0] > 2):
    print("Processing file", i, "out of", num_files, end="\r", flush=True) # python3
  else:
    print("Processing file", i, "out of", num_files, end="\r") # python2
  i += 1

print("\nProcessing completed")

time = data_sum[:num_rows,0]
electric_energy = data_sum[:num_rows,1]
magnetic_energy = data_sum[:num_rows,2]
kinetic_energy_e = data_sum[:num_rows,7]
thermal_energy_e = data_sum[:num_rows,8]
kinetic_energy_i = data_sum[:num_rows,13]
thermal_energy_i = data_sum[:num_rows,14]

energy = electric_energy[:] + magnetic_energy[:] + kinetic_energy_e[:] + \
          thermal_energy_e[:] + kinetic_energy_i[:] + thermal_energy_i[:]
axs[0,1].plot(time[:], energy[:]/energy[0], color="orange", linestyle=":")


axs[0,1].set_xlim([0.,40.])
axs[0,1].legend([r'Moment Fitting $32^3$',r'Standard PFC $32^3$',r'Moment Fitting $18^3$',r'Standard PFC $18^3$'], loc="upper left", prop={'size': 7.25})



axs[0,0].set_title(r'(a) Magnetic Flux') 
axs[0,1].set_title(r'(b) Total Relative Energy')   
axs[1,0].set_title(r'(c) $j_z$ Moment Fitting $32^3$ ($t=21\,\Omega_{i,0}$)') 
axs[1,1].set_title(r'(d) $j_z$ Standard PFC $32^3$ ($t=19\,\Omega_{i,0}$)')   
axs[2,0].set_title(r'(e) $j_z$ Moment Fitting $18^3$ ($t=22\,\Omega_{i,0}$)')
axs[2,1].set_title(r'(f) $j_z$ Standard PFC $18^3$ ($t=18.5\,\Omega_{i,0}$)')

plt.tight_layout()
plt.savefig(writepath, bbox_inches='tight', dpi=300)

#plt.show()
