from __future__ import print_function
import numpy as np
import matplotlib.pyplot as plt
import glob
import sys
from matplotlib.ticker import ScalarFormatter
from matplotlib import rc
from vtk import vtkXMLPImageDataReader
from vtk.util import numpy_support
import pyspedas
from pytplot import tplot
from datetime import datetime, timedelta
import datetime as dati
from spacepy.coordinates import Coords
from spacepy.time import Ticktock
from scipy import interpolate

# n_0     = 0.09*100**3 m^(-3)
# B_0     = 12.e-9 T
# q_0     = e = 1.602176634e-19 C
# d_i,0   = sqrt(m_i/(n_i q_0^2 mu_0)) = sqrt(1.672621898e-27/(0.09*100**3 * (1.6021766208e-19)**2 * 12.566370614e-7))
#         = 759.03589147017063623376763 km
# d_e,0   = sqrt(m_e/(n_e q_0^2 mu_0)) = sqrt(9.10938356e-31/(0.09*100**3 * (1.6021766208e-19)**2 * 12.566370614e-7))
#         = 17.71364421871059690928576 km
# v_A,e,0 = B_0/sqrt(mu_0 * n_e,0 * m_e) = 12e-9/sqrt(12.566370614e-7*(0.09*100**3)*9.10938356e-31)
#         = 3.73861345713986384685e7 m/s = 3.73861345713986384685e4 km/s
# v_A,i,0 = 12e-9/sqrt(12.566370614e-7*(0.09*100**3)*1.672621898e-27)
#         = 872.48138586423806389295797 km/s
# E_0     = B_0 * v_A,e = 12.e-9 T * 872481.38586423806389295797 m/s
#         = 0.01046977663037085677 V/M = 10.46977663037085677 mV/m

def vAe_to_kmpse4(v):
  return v*3.73861345713986384685

def kmpse4_to_vAe(v):
  return v/3.73861345713986384685

def B0_to_nT(B):
  return B*12.

def nT_to_B0(B):
  return B/12.

def E0_to_mVpm(E):
  return E*10.46977663037085677

def mVpm_to_E0(E):
  return E/10.46977663037085677


rc('text', usetex=True)
rc('font', family='serif')

writepath = "magnetotail_mms_data.eps"


fig, axs = plt.subplots(3, 2, figsize=(9.,6.))



### Transformation MMS -> simulation coordinate system from Nakamura/Genestreti papers
# BL = 0.9482*Bx - 0.2551*By - 0.1893*Bz
# BN = 0.2651*Bx + 0.3074*By + 0.9139*Bz
# BM = 0.1749*Bx + 0.9168*By - 0.3591*Bz


### MMS data


# magnetic field
mms_fgm = pyspedas.mms.fgm(trange=['2017-07-11/22:34:00', '2017-07-11/22:34:05'], data_rate='brst', probe=3, notplot=True)

t_fgm = mms_fgm['mms3_fgm_b_gsm_brst_l2']['x']
Bx = mms_fgm['mms3_fgm_b_gsm_brst_l2']['y'][:,0]
By = mms_fgm['mms3_fgm_b_gsm_brst_l2']['y'][:,1]
Bz = mms_fgm['mms3_fgm_b_gsm_brst_l2']['y'][:,2]

t_fgm_conv = [datetime.fromtimestamp(elem) + timedelta(hours=-2) for elem in t_fgm]
BL =   0.9482*Bx - 0.2551*By - 0.1893*Bz
BM =   0.2651*Bx + 0.3074*By + 0.9139*Bz
BN = -(0.1749*Bx + 0.9168*By - 0.3591*Bz)


# electric field
mms_edp = pyspedas.mms.edp(trange=['2017-07-11/22:34:00', '2017-07-11/22:34:05'], probe=3, notplot=True)
t_edp = mms_edp['mms3_edp_dce_gse_fast_l2']['x']

t_edp_conv = [datetime.fromtimestamp(elem) + timedelta(hours=-2) for elem in t_edp]
gse_coords = Coords(mms_edp['mms3_edp_dce_gse_fast_l2']['y'], 'GSE', 'car')
gse_coords.ticks = Ticktock(t_edp_conv, 'ISO')
Ex = gse_coords.convert('GSM', 'car').x
Ey = gse_coords.convert('GSM', 'car').y
Ez = gse_coords.convert('GSM', 'car').z

EL =   0.9482*Ex - 0.2551*Ey - 0.1893*Ez  
EM =   0.2651*Ex + 0.3074*Ey + 0.9139*Ez  
EN = -(0.1749*Ex + 0.9168*Ey - 0.3591*Ez) 


# electron velocity and density
mms_des = pyspedas.mms.fpi(trange=['2017-07-11/22:34:00', '2017-07-11/22:34:05'], datatype='des-moms',
  data_rate='brst', center_measurement=True, probe=3, notplot=True)

t_des_v = mms_des['mms3_des_bulkv_gse_brst']['x']
t_des_v_conv = [datetime.fromtimestamp(elem) + timedelta(hours=-2) for elem in t_des_v]
gse_coords = Coords(mms_des['mms3_des_bulkv_gse_brst']['y'], 'GSE', 'car')
gse_coords.ticks = Ticktock(t_des_v_conv, 'ISO')
vex = gse_coords.convert('GSM', 'car').x
vey = gse_coords.convert('GSM', 'car').y
vez = gse_coords.convert('GSM', 'car').z

veL = (  0.9482*vex - 0.2551*vey - 0.1893*vez )
veM = (  0.2651*vex + 0.3074*vey + 0.9139*vez )
veN = (-(0.1749*vex + 0.9168*vey - 0.3591*vez))



### simulation data

loadpath = 'nakamura_vlasov_1536x512x36xx_m100_gf-m0p03_deltaf_dp/output150.pvti'
reader = vtkXMLPImageDataReader()
print("Path: ", loadpath)

reader.SetFileName(loadpath)
reader.Update()
img = reader.GetOutput()

dims = [img.GetDimensions()[0]-1,img.GetDimensions()[1]-1,img.GetDimensions()[2]-1]
origin = img.GetOrigin()
spacing = img.GetSpacing()

x = np.array([origin[0] + i*spacing[0] for i in range(dims[0])])
y = np.array([origin[1] + i*spacing[1] for i in range(dims[1])])

ux_e = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('ux_e'))
uy_e = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('uy_e'))
uz_e = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('uz_e'))
Bx = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Bx'))
By = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('By'))
Bz = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Bz'))
Ex = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Ex'))
Ey = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Ey'))
Ez = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Ez'))

ux_e = np.reshape(ux_e,(dims[1],dims[0]))
uy_e = np.reshape(uy_e,(dims[1],dims[0]))
uz_e = np.reshape(uz_e,(dims[1],dims[0]))
Bx = np.reshape(Bx,(dims[1],dims[0]))
By = np.reshape(By,(dims[1],dims[0]))
Bz = np.reshape(Bz,(dims[1],dims[0]))
Ex = np.reshape(Ex,(dims[1],dims[0]))
Ey = np.reshape(Ey,(dims[1],dims[0]))
Ez = np.reshape(Ez,(dims[1],dims[0]))

num_line_points = 1000
line_beg_x = -3.95
line_end_x = 2.83
line_beg_y = -0.24
line_end_y = 0.06
line_x = np.linspace(line_beg_x, line_end_x, num_line_points)
line_y = np.linspace(line_beg_y, line_end_y, num_line_points)
path_length = np.sqrt( (line_end_x-line_beg_x)**2 + (line_end_y-line_beg_y)**2 )
r = np.linspace(0.,path_length,num_line_points)

f = interpolate.RectBivariateSpline(y,x,ux_e)
ux_e_sim = f(line_y, line_x)
ux_e_line = np.array([ux_e_sim[i,i] for i in range(num_line_points)])
f = interpolate.RectBivariateSpline(y,x,uy_e)
uy_e_sim = f(line_y, line_x)
uy_e_line = np.array([uy_e_sim[i,i] for i in range(num_line_points)])
f = interpolate.RectBivariateSpline(y,x,uz_e)
uz_e_sim = f(line_y, line_x)
uz_e_line = np.array([uz_e_sim[i,i] for i in range(num_line_points)])
f = interpolate.RectBivariateSpline(y,x,Ex)
Ex_sim = f(line_y, line_x)
Ex_line = np.array([Ex_sim[i,i] for i in range(num_line_points)])
f = interpolate.RectBivariateSpline(y,x,Ey)
Ey_sim = f(line_y, line_x)
Ey_line = np.array([Ey_sim[i,i] for i in range(num_line_points)])
f = interpolate.RectBivariateSpline(y,x,Ez)
Ez_sim = f(line_y, line_x)
Ez_line = np.array([Ez_sim[i,i] for i in range(num_line_points)])
f = interpolate.RectBivariateSpline(y,x,Bx)
Bx_sim = f(line_y, line_x)
Bx_line = np.array([Bx_sim[i,i] for i in range(num_line_points)])
f = interpolate.RectBivariateSpline(y,x,By)
By_sim = f(line_y, line_x)
By_line = np.array([By_sim[i,i] for i in range(num_line_points)])
f = interpolate.RectBivariateSpline(y,x,Bz)
Bz_sim = f(line_y, line_x)
Bz_line = np.array([Bz_sim[i,i] for i in range(num_line_points)])


### plotting

for i in range(3):
  axs[i,0].set_xlim([0., path_length])
  axs[i,1].set_xlim([dati.datetime(2017,7,11,22,34,0), dati.datetime(2017,7,11,22,34,5)])

for i in range(2):
  axs[i,0].xaxis.set_ticklabels([])
  axs[i,1].xaxis.set_ticklabels([])

axs[0,1].plot(t_des_v_conv,veL/1.e4, t_des_v_conv,veM/1.e4, t_des_v_conv,veN/1.e4)
axs[0,1].legend([r'$u_{e,L}$',r'$u_{e,M}$',r'$u_{e,N}$'], loc="upper left", prop={'size': 8.5})
axs[0,1].set_ylabel(r'$\mathbf{u} / \mathrm{(10^4 km/s)}$')
axs[0,1].set_ylim(-1.,2.)

axs[1,1].plot(t_fgm_conv,BL, t_fgm_conv,BM, t_fgm_conv,BN)
axs[1,1].legend([r'$B_L$',r'$B_M$',r'$B_N$'], loc="upper left", prop={'size': 8.5})
axs[1,1].set_ylim(-2.4,1.05)
axs[1,1].set_ylabel(r'$\mathbf{B} / \mathrm{nT}$')

axs[2,1].plot(t_edp_conv,EL, t_edp_conv,EM, t_edp_conv,EN)
axs[2,1].legend([r'$E_L$',r'$E_M$',r'$E_N$'], loc="upper left", prop={'size': 8.5})
axs[2,1].set_ylim(-17,35)
axs[2,1].set_xlabel(r'Time / s from 22:34:00 UT on 11 Jul 2017')
axs[2,1].set_ylabel(r'$\mathbf{E} / (\mathrm{mV/m})$')
axs[2,1].xaxis.set_ticklabels([0,1,2,3,4,5])

axs[0,0].plot(r,ux_e_line*0.1, r,uy_e_line*0.1, r,uz_e_line*0.1) # *0.1 to translate from v_A,i to v_A,e (sqrt(m_e)=0.1 as m_e = 0.01)
axs[0,0].legend([r'$u_{e,x}$',r'$u_{e,y}$',r'$u_{e,z}$'], loc="upper left", prop={'size': 8.5})
axs[0,0].set_ylabel(r'$\mathbf{u} / v_{A,e,0}$')
axs[0,0].set_ylim(-0.45,0.75)
secaxy1 = axs[0,0].secondary_yaxis('right', functions=(vAe_to_kmpse4, kmpse4_to_vAe))

axs[1,0].plot(r,Bx_line, r,By_line, r,Bz_line)
axs[1,0].legend([r'$B_x$',r'$B_y$',r'$B_z$'], loc="upper left", prop={'size': 8.5})
axs[1,0].set_ylabel(r'$\mathbf{B} / B_0$')
axs[1,0].set_ylim(-0.2,0.15)
secaxy2 = axs[1,0].secondary_yaxis('right', functions=(B0_to_nT, nT_to_B0))

axs[2,0].plot(r,Ex_line, r,Ey_line, r,Ez_line)
axs[2,0].legend([r'$E_x$',r'$E_y$',r'$E_z$'], loc="upper left", prop={'size': 8.5})
axs[2,0].set_xlabel(r'$r / d_{i,0}$')
axs[2,0].set_ylabel(r'$\mathbf{E} / (B_0\,v_{A,0})$')
axs[2,0].set_ylim(-0.5,1.)
secaxy3 = axs[2,0].secondary_yaxis('right', functions=(E0_to_mVpm, mVpm_to_E0))

axs[0,0].set_title(r'(a) Simulation Data', fontsize=16)
axs[0,1].set_title(r'(b) MMS Measurements', fontsize=16)

plt.tight_layout()
plt.savefig(writepath, bbox_inches='tight', dpi=300)

