from __future__ import print_function
import numpy as np
import matplotlib.pyplot as plt
import glob
import sys
from matplotlib.ticker import ScalarFormatter
from matplotlib import rc
from vtk import vtkXMLPImageDataReader
from vtk.util import numpy_support

rc('text', usetex=True)
rc('font', family='serif')

writepath = "magnetotail_ux.eps"


fig, axs = plt.subplots(3, 2, figsize=(7.5,6.3))

for i in range(3):
  for j in range(2):
    axs[i,j].set_xlabel(r'$x / d_{i,0}$')

for i in range(3):
  axs[i,0].set_ylabel(r'$y / d_{i,0}$')


######## pcolor

loadpath = 'nakamura_vlasov_1536x512x36xx_m100_gf-m0p03_deltaf_dp/output150.pvti'
reader = vtkXMLPImageDataReader()
print("Path: ", loadpath)

reader.SetFileName(loadpath)
reader.Update()
img = reader.GetOutput()

dims = [img.GetDimensions()[0]-1,img.GetDimensions()[1]-1,img.GetDimensions()[2]-1]
origin = img.GetOrigin()
spacing = img.GetSpacing()

x = np.array([origin[0] + i*spacing[0] for i in range(dims[0])])
y = np.array([origin[1] + i*spacing[1] for i in range(dims[1])])


# electron outflow velocity
field_name = 'ux_e'
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray(field_name))
data = np.reshape(data,(dims[1],dims[0]))
z = np.zeros([dims[1]//8+1,dims[0]//8+1])
z[:,:] = data[int(dims[1]*7/16):int(dims[1]*9/16)+1,int(dims[0]*7/16):int(dims[0]*9/16)+1]
z_min = -4;
z_max = 4;

im = axs[0,0].pcolor(x[int(dims[0]*7/16):int(dims[0]*9/16)+1], y[int(dims[1]*7/16):int(dims[1]*9/16)+1],
  z, cmap='coolwarm', vmin=z_min, vmax=z_max, rasterized=True, shading='auto')
fig.colorbar(im, ax=axs[0,0], pad = 0.3, aspect=40, location='bottom', shrink = .7)

num_line_points = 1000
line_beg_x = -3.95
line_end_x = 2.83
line_beg_y = -0.24
line_end_y = 0.06
line_x = np.linspace(line_beg_x, line_end_x, num_line_points)
line_y = np.linspace(line_beg_y, line_end_y, num_line_points)
axs[0,0].plot(line_x, line_y, color='white', linewidth=0.75)


# ion outflow velocity
field_name = 'ux_i'
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray(field_name))
data = np.reshape(data,(dims[1],dims[0]))
z = np.zeros([dims[1]//2+1,dims[0]//2+1])
z[:,:] = data[int(dims[1]//4):int(dims[1]//4*3)+1,int(dims[0]//4):int(dims[0]//4*3)+1]
z_min = -0.8;
z_max = 0.8;

im = axs[0,1].pcolor(x[int(dims[0]//4):int(dims[0]//4*3)+1], y[int(dims[1]//4):int(dims[1]//4*3)+1],
  z, cmap='coolwarm', vmin=z_min, vmax=z_max, rasterized=True, shading='auto')
cbar = fig.colorbar(im, ax=axs[0,1], pad = 0.3, aspect=40, location='bottom', shrink = .7)
cbar.set_ticks([-.8,-.4,0.,.4,.8])


field_name = 'Bx'
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray(field_name))
Bx = np.reshape(data,(dims[1],dims[0]))
field_name = 'By'
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray(field_name))
By = np.reshape(data,(dims[1],dims[0]))
axs[0,1].streamplot(x[int(dims[0]//4):int(dims[0]//4*3)+1], y[int(dims[1]//4):int(dims[1]//4*3)+1],
  Bx[int(dims[1]//4):int(dims[1]//4*3)+1,int(dims[0]//4):int(dims[0]//4*3)+1],
  By[int(dims[1]//4):int(dims[1]//4*3)+1,int(dims[0]//4):int(dims[0]//4*3)+1], density=(2.5,0.75),
  linewidth=0.2, arrowsize=0.3, color="white", arrowstyle="fancy")


data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('T_e'))
T = np.reshape(data,(dims[1],dims[0]))[int(dims[1]//4):int(dims[1]//4*3)+1,int(dims[0]//4):int(dims[0]//4*3)+1]
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Qxxx_e'))
Qxxx = np.reshape(data,(dims[1],dims[0]))[int(dims[1]//4):int(dims[1]//4*3)+1,int(dims[0]//4):int(dims[0]//4*3)+1]
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Qxxy_e'))
Qxxy = np.reshape(data,(dims[1],dims[0]))[int(dims[1]//4):int(dims[1]//4*3)+1,int(dims[0]//4):int(dims[0]//4*3)+1]
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Qxxz_e'))
Qxxz = np.reshape(data,(dims[1],dims[0]))[int(dims[1]//4):int(dims[1]//4*3)+1,int(dims[0]//4):int(dims[0]//4*3)+1]
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Qxyy_e'))
Qxyy = np.reshape(data,(dims[1],dims[0]))[int(dims[1]//4):int(dims[1]//4*3)+1,int(dims[0]//4):int(dims[0]//4*3)+1]
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Qxyz_e'))
Qxyz = np.reshape(data,(dims[1],dims[0]))[int(dims[1]//4):int(dims[1]//4*3)+1,int(dims[0]//4):int(dims[0]//4*3)+1]
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Qxzz_e'))
Qxzz = np.reshape(data,(dims[1],dims[0]))[int(dims[1]//4):int(dims[1]//4*3)+1,int(dims[0]//4):int(dims[0]//4*3)+1]
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Qyyy_e'))
Qyyy = np.reshape(data,(dims[1],dims[0]))[int(dims[1]//4):int(dims[1]//4*3)+1,int(dims[0]//4):int(dims[0]//4*3)+1]
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Qyyz_e'))
Qyyz = np.reshape(data,(dims[1],dims[0]))[int(dims[1]//4):int(dims[1]//4*3)+1,int(dims[0]//4):int(dims[0]//4*3)+1]
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Qyzz_e'))
Qyzz = np.reshape(data,(dims[1],dims[0]))[int(dims[1]//4):int(dims[1]//4*3)+1,int(dims[0]//4):int(dims[0]//4*3)+1]
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Qzzz_e'))
Qzzz = np.reshape(data,(dims[1],dims[0]))[int(dims[1]//4):int(dims[1]//4*3)+1,int(dims[0]//4):int(dims[0]//4*3)+1]

qx = Qxxx + Qxyy + Qxzz
qy = Qxxy + Qyyy + Qyzz
qz = Qxxz + Qyyz + Qzzz
q = np.sqrt(qx**2 + qy**2 + qz**2)

# electron heat flux scalar
im = axs[1,0].pcolor(x[int(dims[0]//4):int(dims[0]//4*3)+1], y[int(dims[1]//4):int(dims[1]//4*3)+1],
  q, cmap='magma', rasterized=True, shading='auto')
cbar = fig.colorbar(im, ax=axs[1,0], pad = 0.3, aspect=40, location='bottom', shrink = .7)

# electron temperature
im = axs[2,0].pcolor(x[int(dims[0]//4):int(dims[0]//4*3)+1], y[int(dims[1]//4):int(dims[1]//4*3)+1],
  T, cmap='magma', rasterized=True, shading='auto')
cbar = fig.colorbar(im, ax=axs[2,0], pad = 0.3, aspect=40, location='bottom', shrink = .7)

nx = 64
ny = 64
x_new = x[int(dims[0]//4):int(dims[0]//4*3)+1]
y_new = y[int(dims[1]//4):int(dims[1]//4*3)+1]
axs[2,0].quiver(x_new[0:dims[0]//2-1:dims[0]//nx], y_new[0:dims[1]//2-1:dims[1]//ny],
  qx[0:dims[1]//2-1:dims[1]//ny, 0:dims[0]//2-1:dims[0]//nx], qy[0:dims[1]//2-1:dims[1]//ny, 0:dims[0]//2-1:dims[0]//nx],
  color="gray", width=0.002, scale=1.)


data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('T_i'))
T = np.reshape(data,(dims[1],dims[0]))[int(dims[1]//4):int(dims[1]//4*3)+1,int(dims[0]//4):int(dims[0]//4*3)+1]
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Qxxx_i'))
Qxxx = np.reshape(data,(dims[1],dims[0]))[int(dims[1]//4):int(dims[1]//4*3)+1,int(dims[0]//4):int(dims[0]//4*3)+1]
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Qxxy_i'))
Qxxy = np.reshape(data,(dims[1],dims[0]))[int(dims[1]//4):int(dims[1]//4*3)+1,int(dims[0]//4):int(dims[0]//4*3)+1]
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Qxxz_i'))
Qxxz = np.reshape(data,(dims[1],dims[0]))[int(dims[1]//4):int(dims[1]//4*3)+1,int(dims[0]//4):int(dims[0]//4*3)+1]
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Qxyy_i'))
Qxyy = np.reshape(data,(dims[1],dims[0]))[int(dims[1]//4):int(dims[1]//4*3)+1,int(dims[0]//4):int(dims[0]//4*3)+1]
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Qxyz_i'))
Qxyz = np.reshape(data,(dims[1],dims[0]))[int(dims[1]//4):int(dims[1]//4*3)+1,int(dims[0]//4):int(dims[0]//4*3)+1]
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Qxzz_i'))
Qxzz = np.reshape(data,(dims[1],dims[0]))[int(dims[1]//4):int(dims[1]//4*3)+1,int(dims[0]//4):int(dims[0]//4*3)+1]
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Qyyy_i'))
Qyyy = np.reshape(data,(dims[1],dims[0]))[int(dims[1]//4):int(dims[1]//4*3)+1,int(dims[0]//4):int(dims[0]//4*3)+1]
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Qyyz_i'))
Qyyz = np.reshape(data,(dims[1],dims[0]))[int(dims[1]//4):int(dims[1]//4*3)+1,int(dims[0]//4):int(dims[0]//4*3)+1]
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Qyzz_i'))
Qyzz = np.reshape(data,(dims[1],dims[0]))[int(dims[1]//4):int(dims[1]//4*3)+1,int(dims[0]//4):int(dims[0]//4*3)+1]
data = numpy_support.vtk_to_numpy(img.GetCellData().GetArray('Qzzz_i'))
Qzzz = np.reshape(data,(dims[1],dims[0]))[int(dims[1]//4):int(dims[1]//4*3)+1,int(dims[0]//4):int(dims[0]//4*3)+1]

qx = Qxxx + Qxyy + Qxzz
qy = Qxxy + Qyyy + Qyzz
qz = Qxxz + Qyyz + Qzzz
q = np.sqrt(qx**2 + qy**2 + qz**2)

# ion heat flux scalar
im = axs[1,1].pcolor(x[int(dims[0]//4):int(dims[0]//4*3)+1], y[int(dims[1]//4):int(dims[1]//4*3)+1],
  q, cmap='magma', rasterized=True, shading='auto')
cbar = fig.colorbar(im, ax=axs[1,1], pad = 0.3, aspect=40, location='bottom', shrink = .7)

# ion temperature
im = axs[2,1].pcolor(x[int(dims[0]//4):int(dims[0]//4*3)+1], y[int(dims[1]//4):int(dims[1]//4*3)+1],
  T, cmap='magma', rasterized=True, shading='auto')
cbar = fig.colorbar(im, ax=axs[2,1], pad = 0.3, aspect=40, location='bottom', shrink = .7)

nx = 64
ny = 64
x_new = x[int(dims[0]//4):int(dims[0]//4*3)+1]
y_new = y[int(dims[1]//4):int(dims[1]//4*3)+1]
axs[2,1].quiver(x_new[0:dims[0]//2-1:dims[0]//nx], y_new[0:dims[1]//2-1:dims[1]//ny],
  qx[0:dims[1]//2-1:dims[1]//ny, 0:dims[0]//2-1:dims[0]//nx], qy[0:dims[1]//2-1:dims[1]//ny, 0:dims[0]//2-1:dims[0]//nx],
  color="gray", width=0.002, scale=4.)




axs[0,0].set_title(r'(c) Electron Outflow Velocity') 
axs[0,1].set_title(r'(d) Ion Outflow Velocity') 
axs[1,0].set_title(r'(e) Electron Heat Flux') 
axs[1,1].set_title(r'(f) Ion Heat Flux') 
axs[2,0].set_title(r'(g) Electron Temperature') 
axs[2,1].set_title(r'(h) Ion Temperature') 

plt.tight_layout()
plt.savefig(writepath, bbox_inches='tight', dpi=300)

